@@ -1,10 +1,8 @@
 /* SPDX-License-Identifier: MIT */
 
 #include <stdlib.h>
-//#include <elogind/sd-event.h>
-//#include <glib.h>
+#include <elogind/sd-event.h>
 #include <glib.h>
-#include <systemd/sd-event.h>
 
 typedef struct SDEventSource {
   GSource source;
