@@ -95,4 +95,4 @@
                 "/usr/lib/" n                   \
                 _CONF_PATHS_SPLIT_USR(n))
 
-#define LONG_LINE_MAX (1U*1024U*1024U)
+#define HIGH_RLIMIT_MEMLOCK (1024ULL*1024ULL*64ULL)
