@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 #pragma once
 
-
 #include <inttypes.h>
 
 #if SIZEOF_PID_T == 4
@@ -43,6 +42,7 @@
 #  define PRI_TIMEX "li"
 #endif
 
+#ifdef __GLIBC__ /// Go directly for %ju in elogind if not. Enhances musl-libc compatibility.
 #if SIZEOF_RLIM_T == 8
 #  define RLIM_FMT "%" PRIu64
 #elif SIZEOF_RLIM_T == 4
@@ -50,6 +50,9 @@
 #else
 #  error Unknown rlim_t size
 #endif
+#else
+#define RLIM_FMT "%ju"
+#endif // __GLIBC__
 
 #if SIZEOF_DEV_T == 8
 #  define DEV_FMT "%" PRIu64
