@@ -9,7 +9,7 @@ int mount_setup_early(void);
 int mount_setup(bool loaded_policy);
 
 #if 0 /// UNNEEDED by elogind
-int mount_cgroup_controllers(char ***join_controllers);
+int mount_cgroup_controllers(void);
 
 bool mount_point_is_api(const char *path);
 bool mount_point_ignore(const char *path);
