@@ -1,9 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 #pragma once
 
-/***
-***/
-
 #include <byteswap.h>
 #include <stdbool.h>
 #include <sys/socket.h>
@@ -215,8 +212,6 @@ int bus_message_new_synthetic_error(sd_bus *bus, uint64_t serial, const sd_bus_e
 int bus_message_remarshal(sd_bus *bus, sd_bus_message **m);
 
 #if 0 /// UNNEEDED by elogind
-int bus_message_append_sender(sd_bus_message *m, const char *sender);
 #endif // 0
-
 void bus_message_set_sender_driver(sd_bus *bus, sd_bus_message *m);
 void bus_message_set_sender_local(sd_bus *bus, sd_bus_message *m);
