@@ -16,6 +16,7 @@
 #include "audit-util.h"
 #include "bus-error.h"
 #include "bus-util.h"
+#include "env-file.h"
 #include "escape.h"
 #include "fd-util.h"
 #include "fileio.h"
@@ -26,9 +27,11 @@
 #include "parse-util.h"
 #include "path-util.h"
 //#include "process-util.h"
+#include "serialize.h"
 #include "string-table.h"
 //#include "strv.h"
 #include "terminal-util.h"
+#include "tmpfile-util.h"
 #include "user-util.h"
 #include "util.h"
 
@@ -39,6 +42,7 @@
 #define RELEASE_USEC (20*USEC_PER_SEC)
 
 static void session_remove_fifo(Session *s);
+static void session_restore_vt(Session *s);
 
 int session_new(Session **ret, Manager *m, const char *id) {
         _cleanup_(session_freep) Session *s = NULL;
@@ -84,11 +88,10 @@ int session_new(Session **ret, Manager *m, const char *id) {
 Session* session_free(Session *s) {
         SessionDevice *sd;
 
+        log_debug_elogind("Freeing Session %s ...", s->id);
         if (!s)
                 return NULL;
 
-        log_debug_elogind("Freeing Session %s ...", s->id);
-
         if (s->in_gc_queue)
                 LIST_REMOVE(gc_queue, s->manager->session_gc_queue, s);
 
@@ -417,7 +420,7 @@ int session_load(Session *s) {
 
         assert(s);
 
-        r = parse_env_file(NULL, s->state_file, NEWLINE,
+        r = parse_env_file(NULL, s->state_file,
                            "REMOTE",         &remote,
                            "SCOPE",          &s->scope,
 #if 0 /// elogind does not support systemd scope_jobs
@@ -444,8 +447,7 @@ int session_load(Session *s) {
                            "CONTROLLER",     &controller,
                            "ACTIVE",         &active,
                            "DEVICES",        &devices,
-                           "IS_DISPLAY",     &is_display,
-                           NULL);
+                           "IS_DISPLAY",     &is_display);
 
         if (r < 0)
                 return log_error_errno(r, "Failed to read %s: %m", s->state_file);
@@ -454,10 +456,10 @@ int session_load(Session *s) {
                 uid_t u;
                 User *user;
 
-                if (!uid) {
-                        log_error("UID not specified for session %s", s->id);
-                        return -ENOENT;
-                }
+                if (!uid)
+                        return log_error_errno(SYNTHETIC_ERRNO(ENOENT),
+                                               "UID not specified for session %s",
+                                               s->id);
 
                 r = parse_uid(uid, &u);
                 if (r < 0)  {
@@ -466,10 +468,10 @@ int session_load(Session *s) {
                 }
 
                 user = hashmap_get(s->manager->users, UID_TO_PTR(u));
-                if (!user) {
-                        log_error("User of session %s not known.", s->id);
-                        return -ENOENT;
-                }
+                if (!user)
+                        return log_error_errno(SYNTHETIC_ERRNO(ENOENT),
+                                               "User of session %s not known.",
+                                               s->id);
 
                 log_debug_elogind("Attaching session %s to user %d", s->id, user->uid);
                 session_set_user(s, user);
@@ -498,7 +500,7 @@ int session_load(Session *s) {
                 s->vtnr = 0;
 
         if (position && s->seat) {
-                unsigned int npos;
+                unsigned npos;
 
                 safe_atou(position, &npos);
                 seat_claim_position(s->seat, s, npos);
@@ -568,7 +570,6 @@ int session_load(Session *s) {
                         s->fifo_path = mfree(s->fifo_path);
                 }
 #endif // 1
-
                 /* If we open an unopened pipe for reading we will not
                    get an EOF. to trigger an EOF we hence open it for
                    writing, but close it right away which then will
@@ -581,9 +582,9 @@ int session_load(Session *s) {
         }
 
         if (realtime)
-                timestamp_deserialize(realtime, &s->timestamp.realtime);
+                (void) deserialize_usec(realtime, &s->timestamp.realtime);
         if (monotonic)
-                timestamp_deserialize(monotonic, &s->timestamp.monotonic);
+                (void) deserialize_usec(monotonic, &s->timestamp.monotonic);
 
         if (active) {
                 k = parse_boolean(active);
@@ -615,7 +616,7 @@ int session_load(Session *s) {
 }
 
 int session_activate(Session *s) {
-        unsigned int num_pending;
+        unsigned num_pending;
 
         assert(s);
         assert(s->user);
@@ -850,13 +851,12 @@ int session_stop(Session *s, bool force) {
 
         if (!s->user)
                 return -ESTALE;
+        log_debug_elogind("Stopping session %s %s ...", s->id, force ? "(forced)" : "");
         if (!s->started)
                 return 0;
         if (s->stopping)
                 return 0;
 
-        log_debug_elogind("Stopping session %s %s ...", s->id, force ? "(forced)" : "");
-
         s->timer_event_source = sd_event_source_unref(s->timer_event_source);
 
         if (s->seat)
@@ -898,7 +898,6 @@ int session_finalize(Session *s) {
                            "USER_ID=%s", s->user->name,
                            "LEADER="PID_FMT, s->leader,
                            LOG_MESSAGE("Removed session %s.", s->id));
-        else
                 log_debug_elogind("Session %s not started, finalizing...", s->id);
 
         s->timer_event_source = sd_event_source_unref(s->timer_event_source);
@@ -939,7 +938,6 @@ static int release_timeout_callback(sd_event_source *es, uint64_t usec, void *us
         assert(s);
 
         log_debug_elogind("Session release timeout reached, stopping session %s", s->id);
-
         session_stop(s, false);
         return 0;
 }
@@ -952,7 +950,6 @@ int session_release(Session *s) {
                           s->stopping ? "No, already stopping." :
                           s->timer_event_source ? "No, timer already running." :
                           "Yes, releasing in 20 seconds."); /* RELEASE_USEC is a formula :( */
-
         if (!s->started || s->stopping)
                 return 0;
 
@@ -1109,7 +1106,6 @@ static int session_dispatch_fifo(sd_event_source *es, int fd, uint32_t revents,
         /* EOF on the FIFO means the session died abnormally. */
 
         log_debug_elogind("EOF on Session %s FIFO: Session died abnormally, stopping...", s->id);
-
         session_remove_fifo(s);
         session_stop(s, false);
 
@@ -1186,10 +1182,10 @@ static void session_remove_fifo(Session *s) {
                                           current_fifo_fd, s->fifo_path, s->id);
 #endif // 1
                 (void) unlink(s->fifo_path);
+                s->fifo_path = mfree(s->fifo_path);
 #if 1 /// end elogind extra if
                 }
 #endif // 1
-                s->fifo_path = mfree(s->fifo_path);
         }
 }
 
@@ -1208,7 +1204,6 @@ bool session_may_gc(Session *s, bool drop_not_started) {
                           s->fifo_fd < 0           ? "No FIFO opened"   :
                           pipe_eof(s->fifo_fd) > 0 ? "FIFO reached EOF" :
                           "FIFO up and running!");
-
         if (drop_not_started && !s->started)
                 return true;
 
@@ -1377,13 +1372,8 @@ error:
         return r;
 }
 
-void session_restore_vt(Session *s) {
-
-        static const struct vt_mode mode = {
-                .mode = VT_AUTO,
-        };
-
-        int vt, old_fd;
+static void session_restore_vt(Session *s) {
+        int r, vt, old_fd;
 
         /* We need to get a fresh handle to the virtual terminal,
          * since the old file-descriptor is potentially in a hung-up
@@ -1399,12 +1389,9 @@ void session_restore_vt(Session *s) {
         if (vt < 0)
                 return;
 
-        (void) ioctl(vt, KDSETMODE, KD_TEXT);
-
-        (void) vt_reset_keyboard(vt);
-
-        (void) ioctl(vt, VT_SETMODE, &mode);
-        (void) fchown(vt, 0, (gid_t) -1);
+        r = vt_restore(vt);
+        if (r < 0)
+                log_warning_errno(r, "Failed to restore VT, ignoring: %m");
 
         s->vtfd = safe_close(s->vtfd);
 }
@@ -1429,9 +1416,9 @@ void session_leave_vt(Session *s) {
                 return;
 
         session_device_pause_all(s);
-        r = ioctl(s->vtfd, VT_RELDISP, 1);
+        r = vt_release(s->vtfd, false);
         if (r < 0)
-                log_debug_errno(errno, "Cannot release VT of session %s: %m", s->id);
+                log_debug_errno(r, "Cannot release VT of session %s: %m", s->id);
 }
 
 bool session_is_controller(Session *s, const char *sender) {
