@@ -3,15 +3,11 @@
 
 #include <stdbool.h>
 
-#if 0 /// elogind needs the systems udev header
-#include "libudev.h"
-#else
-#include <libudev.h>
-#endif // 0
 #include "sd-bus.h"
+#include "sd-device.h"
 #include "sd-event.h"
 
-//#include "conf-parser.h"
+#include "conf-parser.h"
 #include "hashmap.h"
 #include "list.h"
 #include "set.h"
@@ -25,10 +21,12 @@ typedef struct Manager Manager;
 
 /// Additional includes needed by elogind
 #include "cgroup-util.h"
+#include "elogind.h"
 
 #if 1 /// elogind has to ident itself
-#define MANAGER_IS_SYSTEM(m) ((m)->is_system)
-#define MANAGER_IS_USER(m) (!((m)->is_system))
+#define MANAGER_IS_SYSTEM(m)   (  (m)->is_system)
+#define MANAGER_IS_TEST_RUN(m) (  (m)->test_run_flags != 0)
+#define MANAGER_IS_USER(m)     (!((m)->is_system))
 #endif // 1
 struct Manager {
         sd_event *event;
@@ -46,22 +44,17 @@ struct Manager {
         LIST_HEAD(Session, session_gc_queue);
         LIST_HEAD(User, user_gc_queue);
 
-        struct udev *udev;
-        struct udev_monitor *udev_seat_monitor, *udev_device_monitor, *udev_vcsa_monitor, *udev_button_monitor;
+        sd_device_monitor *device_seat_monitor, *device_monitor, *device_vcsa_monitor, *device_button_monitor;
 
         sd_event_source *console_active_event_source;
-        sd_event_source *udev_seat_event_source;
-        sd_event_source *udev_device_event_source;
-        sd_event_source *udev_vcsa_event_source;
-        sd_event_source *udev_button_event_source;
 
 #if ENABLE_UTMP
         sd_event_source *utmp_event_source;
 #endif
 
-#if 0 /// elogind does not support autospawning of vts
         int console_active_fd;
 
+#if 0 /// elogind does not support autospawning of vts
         unsigned n_autovts;
 
         unsigned reserve_vt;
@@ -78,12 +71,11 @@ struct Manager {
         /* Flags */
         unsigned test_run_flags;
         bool is_system:1; /* true if elogind is its own cgroups manager */
+        bool do_interrupt:1;  /* true if SIGINT is used to stop elogind. See elogind_signal_handler() */
 
         /* Data specific to the cgroup subsystem */
         CGroupMask cgroup_supported;
         char *cgroup_root;
-
-        int console_active_fd;
 #endif // 0
 
         Seat *seat0;
@@ -116,6 +108,9 @@ struct Manager {
          * the job of it */
         char *action_job;
 #else
+        /* Suspension and hibernation can be disabled in logind.conf. */
+        bool allow_suspend, allow_hibernation, allow_suspend_then_hibernate, allow_hybrid_sleep;
+
         /* If an admin puts scripts into SYSTEM_SLEEP_PATH and/or
            SYSTEM_POWEROFF_PATH that fail, the ongoing suspend/poweroff
            action will be cancelled if any of these are set to true. */
@@ -190,12 +185,12 @@ int manager_add_user_by_name(Manager *m, const char *name, User **_user);
 int manager_add_user_by_uid(Manager *m, uid_t uid, User **_user);
 int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **_inhibitor);
 
-int manager_process_seat_device(Manager *m, struct udev_device *d);
-int manager_process_button_device(Manager *m, struct udev_device *d);
+int manager_process_seat_device(Manager *m, sd_device *d);
+int manager_process_button_device(Manager *m, sd_device *d);
 
 #if 0 /// UNNEEDED by elogind
-int manager_spawn_autovt(Manager *m, unsigned int vtnr);
 #endif // 0
+int manager_spawn_autovt(Manager *m, unsigned vtnr);
 
 bool manager_shall_kill(Manager *m, const char *user);
 
@@ -204,6 +199,7 @@ int manager_get_idle_hint(Manager *m, dual_timestamp *t);
 int manager_get_user_by_pid(Manager *m, pid_t pid, User **user);
 int manager_get_session_by_pid(Manager *m, pid_t pid, Session **session);
 
+bool manager_is_lid_closed(Manager *m);
 bool manager_is_docked_or_external_displays(Manager *m);
 bool manager_is_on_external_power(void);
 bool manager_all_buttons_ignored(Manager *m);
