@@ -9,11 +9,11 @@
 //#include <stddef.h>
 //#include <stdio.h>
 //#include <string.h>
+#include <sys/ioctl.h>
 //#include <syslog.h>
 //#include <unistd.h>
 
 #include "alloc-util.h"
-//#include "bootspec.h"
 //#include "conf-parser.h"
 //#include "def.h"
 //#include "env-util.h"
@@ -23,7 +23,6 @@
 //#include "macro.h"
 #include "parse-util.h"
 #include "path-util.h"
-#include "proc-cmdline.h"
 #include "sleep-config.h"
 #include "string-util.h"
 #include "strv.h"
@@ -71,7 +70,7 @@ int parse_sleep_config(const char *verb, bool *ret_allow, char ***ret_modes, cha
                 if (suspend_state)
                         states = TAKE_PTR(suspend_state);
                 else
-                        states = strv_new("mem", "standby", "freeze", NULL);
+                        states = strv_new("mem", "standby", "freeze");
 
         } else if (streq(verb, "hibernate")) {
                 allow = allow_hibernate != 0;
@@ -79,12 +78,12 @@ int parse_sleep_config(const char *verb, bool *ret_allow, char ***ret_modes, cha
                 if (hibernate_mode)
                         modes = TAKE_PTR(hibernate_mode);
                 else
-                        modes = strv_new("platform", "shutdown", NULL);
+                        modes = strv_new("platform", "shutdown");
 
                 if (hibernate_state)
                         states = TAKE_PTR(hibernate_state);
                 else
-                        states = strv_new("disk", NULL);
+                        states = strv_new("disk");
 
         } else if (streq(verb, "hybrid-sleep")) {
                 allow = allow_hybrid_sleep > 0 ||
@@ -93,12 +92,12 @@ int parse_sleep_config(const char *verb, bool *ret_allow, char ***ret_modes, cha
                 if (hybrid_mode)
                         modes = TAKE_PTR(hybrid_mode);
                 else
-                        modes = strv_new("suspend", "platform", "shutdown", NULL);
+                        modes = strv_new("suspend", "platform", "shutdown");
 
                 if (hybrid_state)
                         states = TAKE_PTR(hybrid_state);
                 else
-                        states = strv_new("disk", NULL);
+                        states = strv_new("disk");
 
         } else if (streq(verb, "suspend-then-hibernate")) {
                 allow = allow_s2h > 0 ||
@@ -169,12 +168,16 @@ int can_sleep_disk(char **types) {
                 return true;
 
         /* If /sys is read-only we cannot sleep */
-        if (access("/sys/power/disk", W_OK) < 0)
+        if (access("/sys/power/disk", W_OK) < 0) {
+                log_debug_errno(errno, "/sys/power/disk is not writable: %m");
                 return false;
+        }
 
         r = read_one_line_file("/sys/power/disk", &p);
-        if (r < 0)
+        if (r < 0) {
+                log_debug_errno(r, "Couldn't read /sys/power/disk: %m");
                 return false;
+        }
 
         STRV_FOREACH(type, types) {
                 const char *word, *state;
@@ -259,8 +262,8 @@ int find_hibernate_location(char **device, char **type, size_t *size, size_t *us
                 return 0;
         }
 
-        log_debug("No swap partitions were found.");
-        return -ENOSYS;
+        return log_debug_errno(SYNTHETIC_ERRNO(ENOSYS),
+                               "No swap partitions were found.");
 }
 
 static bool enough_swap_for_hibernation(void) {
@@ -280,14 +283,13 @@ static bool enough_swap_for_hibernation(void) {
 
         r = get_proc_field("/proc/meminfo", "Active(anon)", WHITESPACE, &active);
         if (r < 0) {
-                log_error_errno(r, "Failed to retrieve Active(anon) from /proc/meminfo: %m");
+                log_debug_errno(r, "Failed to retrieve Active(anon) from /proc/meminfo: %m");
                 return false;
         }
 
         r = safe_atollu(active, &act);
         if (r < 0) {
-                log_error_errno(r, "Failed to parse Active(anon) from /proc/meminfo: %s: %m",
-                                active);
+                log_debug_errno(r, "Failed to parse Active(anon) from /proc/meminfo: %s: %m", active);
                 return false;
         }
 
@@ -298,90 +300,6 @@ static bool enough_swap_for_hibernation(void) {
         return r;
 }
 
-#if 0 /// elogind is not init and can not check boot devices and partitions.
-static int check_resume_keys(const char *key, const char *value, void *data) {
-        assert_se(key);
-        assert_se(data);
-
-        int *resume = data;
-
-        if (*resume == 0)
-                /* Exit if we already know we can't resume. */
-                return 0;
-
-        if (streq(key, "noresume")) {
-                log_debug("Found \"noresume\" on the kernel command line, hibernation is disabled.");
-                *resume = 0;
-
-        } else if (streq(key, "resume")) {
-                log_debug("Found resume= option on the kernel command line, hibernation is possible.");
-                *resume = 1;
-        }
-
-        return 0;
-}
-
-static int resume_configured_in_options(const char *options) {
-        int resume = -1, r;
-
-        /* We don't use PROC_CMDLINE_STRIP_RD_PREFIX here, so rd.resume is *not* supported. */
-        r = proc_cmdline_parse_given(options, check_resume_keys, &resume, 0);
-        if (r < 0)
-                return r;
-
-        if (resume < 0)
-                log_debug("Couldn't find resume= option, hibernation is disabled.");
-        return resume > 0;
-}
-
-static int resume_configured(void) {
-        _cleanup_(boot_config_free) BootConfig config = {};
-        const BootEntry *e;
-        int r;
-
-        /* Check whether a valid resume= option is present. If possible, we query the boot options
-         * for the default kernel. If the system is not using sd-boot, fall back to checking the
-         * current kernel command line. This is not perfect, but should suffice for most cases. */
-
-        r = find_default_boot_entry(NULL, NULL, &config, &e);
-        if (r == -ENOKEY)
-                log_debug_errno(r, "Cannot find the ESP partition mount point, falling back to other checks.");
-        else if (r < 0)
-                return log_debug_errno(r, "Cannot read boot configuration from ESP, assuming hibernation is not possible.");
-        else {
-                _cleanup_free_ char *options = NULL;
-
-                options = strv_join(e->options, " ");
-                if (!options)
-                        return log_oom();
-
-                r = resume_configured_in_options(options);
-                if (r < 0)
-                        return log_error_errno(r, "Failed to parse kernel options in \"%s\": %m",
-                                               strnull(e->path));
-                return r;
-        }
-
-        /* If we can't figure out the default boot entry, let's fall back to current kernel cmdline */
-        _cleanup_free_ char *line = NULL;
-        r = proc_cmdline(&line);
-        if (IN_SET(r, -EPERM, -EACCES, -ENOENT))
-                log_debug_errno(r, "Cannot access /proc/cmdline: %m");
-        else if (r < 0)
-                return log_error_errno(r, "Failed to query /proc/cmdline: %m");
-        else {
-                r = resume_configured_in_options(line);
-                if (r < 0)
-                        return log_error_errno(r, "Failed to parse kernel proc cmdline: %m");
-
-                return r;
-        }
-
-        log_debug("Couldn't detect any resume mechanism, hibernation is disabled.");
-        return false;
-}
-#endif // 0
-
 int read_fiemap(int fd, struct fiemap **ret) {
         _cleanup_free_ struct fiemap *fiemap = NULL, *result_fiemap = NULL;
         struct stat statinfo;
@@ -491,7 +409,7 @@ static bool can_s2h(Manager *m) {
 #if 0 /// elogind must transport a pointer to its managers instance
                 r = can_sleep_internal(p, false);
 #else
-                r = can_sleep(m, p);
+                r = can_sleep_internal(m, p, false);
 #endif // 0
                 if (IN_SET(r, 0, -ENOSPC, -EADV)) {
                         log_debug("Unable to %s system.", p);
@@ -507,38 +425,53 @@ static bool can_s2h(Manager *m) {
 
 #if 0 /// elogind has to ask the manager for some stuff
 static int can_sleep_internal(const char *verb, bool check_allowed) {
-        bool allow;
-        _cleanup_strv_free_ char **modes = NULL, **states = NULL;
-        int r;
-
-        assert(STR_IN_SET(verb, "suspend", "hibernate", "hybrid-sleep", "suspend-then-hibernate"));
 #else
 static int can_sleep_internal(Manager *m, const char *verb, bool check_allowed) {
         assert(m);
-
-        char **modes  = streq(verb, "suspend")   ? m->suspend_mode     :
-                        streq(verb, "hibernate") ? m->hibernate_mode   :
-                                                       m->hybrid_sleep_mode;
-        char **states = streq(verb, "suspend")   ? m->suspend_state     :
-                        streq(verb, "hibernate") ? m->hibernate_state   :
-                                                   m->hybrid_sleep_state;
 #endif // 0
+        bool allow;
+#if 0 /// elogind has this parsed already
+        _cleanup_strv_free_ char **modes = NULL, **states = NULL;
+        int r;
+#else
+        char **modes  = streq(verb, "hibernate")    ? m->hibernate_mode    :
+                        streq(verb, "hybrid-sleep") ? m->hybrid_sleep_mode :
+                                                      m->suspend_mode;
+        char **states = streq(verb, "hibernate")    ? m->hibernate_state    :
+                        streq(verb, "hybrid-sleep") ? m->hybrid_sleep_state :
+                                                      m->suspend_state;
+#endif // 0
+
+        assert(STR_IN_SET(verb, "suspend", "hibernate", "hybrid-sleep", "suspend-then-hibernate"));
 
 #if 0 /// already parsed by elogind config
         r = parse_sleep_config(verb, &allow, &modes, &states, NULL);
         if (r < 0)
                 return false;
+#else
+        allow = false;
+        if ( (streq(verb, "suspend")                && m->allow_suspend)                || 
+             (streq(verb, "hibernate")              && m->allow_hibernation)            ||
+             (streq(verb, "suspend-then-hibernate") && m->allow_suspend_then_hibernate) ||
+             (streq(verb, "hybrid-sleep")           && m->allow_hybrid_sleep) )
+                allow = true;
+#endif // 0
 
         if (check_allowed && !allow) {
-#else
-        if (check_allowed && !STR_IN_SET(verb, "suspend", "hibernate", "hybrid-sleep", "suspend-then-hibernate")) {
-#endif // 0
+#if 0 /// be a bit more verbose in elogind
                 log_debug("Sleep mode \"%s\" is disabled by configuration.", verb);
+#else
+                log_info("Sleep mode \"%s\" is disabled by configuration.", verb);
+                log_debug("allow_suspend               : %d", m->allow_suspend);
+                log_debug("allow_hibernation           : %d", m->allow_hibernation);
+                log_debug("allow_hybrid_sleep          : %d", m->allow_hybrid_sleep);
+                log_debug("allow_suspend_then_hibernate: %d", m->allow_suspend_then_hibernate);
+#endif // 0
                 return false;
         }
 
         if (streq(verb, "suspend-then-hibernate"))
-#if 0 /// elogind must transport a pointer to its managers instance
+#if 0 /// elogind can ask its manager.
                 return can_s2h();
 #else
                 return can_s2h(m);
@@ -553,13 +486,6 @@ static int can_sleep_internal(Manager *m, const char *verb, bool check_allowed)
         if (!enough_swap_for_hibernation())
                 return -ENOSPC;
 
-#if 0 /// elogind is not init and can not check this
-        r = resume_configured();
-        if (r <= 0)
-                /* We squash all errors (e.g. EPERM) into a single value for reporting. */
-                return -EADV;
-#endif // 0
-
         return true;
 }
 
