@@ -4,24 +4,26 @@
   Copyright © 2018 Dell Inc.
 ***/
 
-//#include <errno.h>
-//#include <getopt.h>
+#include <errno.h>
+#include <getopt.h>
 #include <linux/fiemap.h>
-//#include <stdio.h>
+#include <stdio.h>
 
 #include "sd-messages.h"
 
-#include "parse-util.h"
 #include "def.h"
 //#include "exec-util.h"
 #include "fd-util.h"
 #include "fileio.h"
 //#include "log.h"
+//#include "main-func.h"
+#include "parse-util.h"
+//#include "pretty-print.h"
 #include "sleep-config.h"
 #include "stdio-util.h"
 #include "string-util.h"
 #include "strv.h"
-//#include "util.h"
+#include "util.h"
 
 /// Additional includes needed by elogind
 #include "exec-elogind.h"
@@ -33,6 +35,9 @@ static char* arg_verb = NULL;
 static int write_hibernate_location_info(void) {
         _cleanup_free_ char *device = NULL, *type = NULL;
         _cleanup_free_ struct fiemap *fiemap = NULL;
+#if 1 /// To support LVM setups, elogind uses device numbers
+        char device_num_str [DECIMAL_STR_MAX(uint32_t) * 2 + 2];
+#endif // 1
         char offset_str[DECIMAL_STR_MAX(uint64_t)];
         char device_str[DECIMAL_STR_MAX(uint64_t)];
         _cleanup_close_ int fd = -1;
@@ -45,22 +50,39 @@ static int write_hibernate_location_info(void) {
                 return log_debug_errno(r, "Unable to find hibernation location: %m");
 
         /* if it's a swap partition, we just write the disk to /sys/power/resume */
-        if (streq(type, "partition"))
-                return write_string_file("/sys/power/resume", device, 0);
-        else if (!streq(type, "file"))
-                return log_debug_errno(EINVAL, "Invalid hibernate type %s: %m",
-                                       type);
+        if (streq(type, "partition")) {
+                r = write_string_file("/sys/power/resume", device, WRITE_STRING_FILE_DISABLE_BUFFER);
+
+#if 1 /// To support LVM setups, elogind uses device numbers if the direct approach failed
+                if (r < 0) {
+                        r = stat(device, &stb);
+                        if (r < 0)
+                                return log_debug_errno(errno, "Error while trying to get stats for %s: %m", device);
+
+                        (void) snprintf(device_num_str, DECIMAL_STR_MAX(uint32_t) * 2 + 2,
+                                        "%u:%u",
+                                        major(stb.st_rdev), minor(stb.st_rdev));
+                        r = write_string_file("/sys/power/resume", device_num_str, 0);
+                }
+#endif // 1
+                if (r < 0)
+                        return log_debug_errno(r, "Failed to write partition device to /sys/power/resume: %m");
+
+                return r;
+        }
+        if (!streq(type, "file"))
+                return log_debug_errno(SYNTHETIC_ERRNO(EINVAL),
+                                       "Invalid hibernate type: %s", type);
 
         /* Only available in 4.17+ */
-        if (access("/sys/power/resume_offset", F_OK) < 0) {
-                if (errno == ENOENT)
+        if (access("/sys/power/resume_offset", W_OK) < 0) {
+                if (errno == ENOENT) {
+                        log_debug("Kernel too old, can't configure resume offset, ignoring.");
                         return 0;
-                return log_debug_errno(errno, "/sys/power/resume_offset unavailable: %m");
-        }
+                }
 
-        r = access("/sys/power/resume_offset", W_OK);
-        if (r < 0)
                 return log_debug_errno(errno, "/sys/power/resume_offset not writeable: %m");
+        }
 
         fd = open(device, O_RDONLY | O_CLOEXEC | O_NONBLOCK);
         if (fd < 0)
@@ -68,26 +90,25 @@ static int write_hibernate_location_info(void) {
         r = fstat(fd, &stb);
         if (r < 0)
                 return log_debug_errno(errno, "Unable to stat %s: %m", device);
+
         r = read_fiemap(fd, &fiemap);
         if (r < 0)
-                return log_debug_errno(r, "Unable to read extent map for '%s': %m",
-                                       device);
-        if (fiemap->fm_mapped_extents == 0) {
-                log_debug("No extents found in '%s'", device);
-                return -EINVAL;
-        }
+                return log_debug_errno(r, "Unable to read extent map for '%s': %m", device);
+        if (fiemap->fm_mapped_extents == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EINVAL),
+                                       "No extents found in '%s'", device);
+
         offset = fiemap->fm_extents[0].fe_physical / page_size();
         xsprintf(offset_str, "%" PRIu64, offset);
-        r = write_string_file("/sys/power/resume_offset", offset_str, 0);
+        r = write_string_file("/sys/power/resume_offset", offset_str, WRITE_STRING_FILE_DISABLE_BUFFER);
         if (r < 0)
-                return log_debug_errno(r, "Failed to write offset '%s': %m",
-                                       offset_str);
+                return log_debug_errno(r, "Failed to write offset '%s': %m", offset_str);
 
         xsprintf(device_str, "%lx", (unsigned long)stb.st_dev);
-        r = write_string_file("/sys/power/resume", device_str, 0);
+        r = write_string_file("/sys/power/resume", device_str, WRITE_STRING_FILE_DISABLE_BUFFER);
         if (r < 0)
-                return log_debug_errno(r, "Failed to write device '%s': %m",
-                                       device_str);
+                return log_debug_errno(r, "Failed to write device '%s': %m", device_str);
+
         return 0;
 }
 
@@ -98,13 +119,12 @@ static int write_mode(char **modes) {
         STRV_FOREACH(mode, modes) {
                 int k;
 
-                k = write_string_file("/sys/power/disk", *mode, 0);
-                if (k == 0)
+                k = write_string_file("/sys/power/disk", *mode, WRITE_STRING_FILE_DISABLE_BUFFER);
+                if (k >= 0)
                         return 0;
 
-                log_debug_errno(k, "Failed to write '%s' to /sys/power/disk: %m",
-                                *mode);
-                if (r == 0)
+                log_debug_errno(k, "Failed to write '%s' to /sys/power/disk: %m", *mode);
+                if (r >= 0)
                         r = k;
         }
 
@@ -118,12 +138,11 @@ static int write_state(FILE **f, char **states) {
         STRV_FOREACH(state, states) {
                 int k;
 
-                k = write_string_stream(*f, *state, 0);
-                if (k == 0)
+                k = write_string_stream(*f, *state, WRITE_STRING_FILE_DISABLE_BUFFER);
+                if (k >= 0)
                         return 0;
-                log_debug_errno(k, "Failed to write '%s' to /sys/power/state: %m",
-                                *state);
-                if (r == 0)
+                log_debug_errno(k, "Failed to write '%s' to /sys/power/state: %m", *state);
+                if (r >= 0)
                         r = k;
 
                 fclose(*f);
@@ -179,6 +198,8 @@ static int execute(Manager *m, const char *verb) {
         if (!f)
                 return log_error_errno(errno, "Failed to open /sys/power/state: %m");
 
+        setvbuf(f, NULL, _IONBF, 0);
+
         /* Configure the hibernation mode */
         if (!strv_isempty(modes)) {
                 r = write_hibernate_location_info();
@@ -186,7 +207,7 @@ static int execute(Manager *m, const char *verb) {
                         return log_error_errno(r, "Failed to write hibernation disk offset: %m");
                 r = write_mode(modes);
                 if (r < 0)
-                        return log_error_errno(r, "Failed to write mode to /sys/power/disk: %m");
+                        return log_error_errno(r, "Failed to write mode to /sys/power/disk: %m");;
         }
 
 #if 0 /// elogind needs its own callbacks to enable cancellation by erroneous scripts
@@ -242,26 +263,30 @@ static int execute(Manager *m, const char *verb) {
         return r;
 }
 
-static int read_wakealarm(uint64_t *result) {
+static int rtc_read_time(uint64_t *ret_sec) {
         _cleanup_free_ char *t = NULL;
-
-        if (read_one_line_file("/sys/class/rtc/rtc0/since_epoch", &t) >= 0)
-                return safe_atou64(t, result);
-        return -EBADF;
-}
-
-static int write_wakealarm(const char *str) {
-
-        _cleanup_fclose_ FILE *f = NULL;
         int r;
 
-        f = fopen("/sys/class/rtc/rtc0/wakealarm", "we");
-        if (!f)
-                return log_error_errno(errno, "Failed to open /sys/class/rtc/rtc0/wakealarm: %m");
-
-        r = write_string_stream(f, str, 0);
+        r = read_one_line_file("/sys/class/rtc/rtc0/since_epoch", &t);
         if (r < 0)
-                return log_error_errno(r, "Failed to write '%s' to /sys/class/rtc/rtc0/wakealarm: %m", str);
+                return log_error_errno(r, "Failed to read RTC time: %m");
+
+        r = safe_atou64(t, ret_sec);
+        if (r < 0)
+                return log_error_errno(r, "Failed to parse RTC time '%s': %m", t);
+
+        return 0;
+}
+
+static int rtc_write_wake_alarm(uint64_t sec) {
+        char buf[DECIMAL_STR_MAX(uint64_t)];
+        int r;
+
+        xsprintf(buf, "%" PRIu64, sec);
+
+        r = write_string_file("/sys/class/rtc/rtc0/wakealarm", buf, WRITE_STRING_FILE_DISABLE_BUFFER);
+        if (r < 0)
+                return log_error_errno(r, "Failed to write '%s' to /sys/class/rtc/rtc0/wakealarm: %m", buf);
 
         return 0;
 }
@@ -277,8 +302,7 @@ static int execute_s2h(Manager *m) {
 
         usec_t hibernate_delay_sec = m->hibernate_delay_sec;
 #endif // 0
-        usec_t orig_time, cmp_time;
-        char time_str[DECIMAL_STR_MAX(uint64_t)];
+        usec_t original_time, wake_time, cmp_time;
         int r;
 
 #if 0 /// Already parsed by elogind config
@@ -291,18 +315,16 @@ static int execute_s2h(Manager *m) {
                 return r;
 #endif // 0
 
-        r = read_wakealarm(&orig_time);
-        if (r < 0)
-                return log_error_errno(errno, "Failed to read time: %d", r);
-
-        orig_time += hibernate_delay_sec / USEC_PER_SEC;
-        xsprintf(time_str, "%" PRIu64, orig_time);
-
-        r = write_wakealarm(time_str);
+        r = rtc_read_time(&original_time);
         if (r < 0)
                 return r;
 
-        log_debug("Set RTC wake alarm for %s", time_str);
+        wake_time = original_time + DIV_ROUND_UP(hibernate_delay_sec, USEC_PER_SEC);
+        r = rtc_write_wake_alarm(wake_time);
+        if (r < 0)
+                return r;
+
+        log_debug("Set RTC wake alarm for %" PRIu64, wake_time);
 
 #if 0 /// elogind uses its manager instance values
         r = execute(suspend_modes, suspend_states);
@@ -312,41 +334,67 @@ static int execute_s2h(Manager *m) {
         if (r < 0)
                 return r;
 
-        r = read_wakealarm(&cmp_time);
+        /* Reset RTC right-away */
+        r = rtc_write_wake_alarm(0);
         if (r < 0)
-                return log_error_errno(errno, "Failed to read time: %d", r);
+                return r;
 
-        /* reset RTC */
-        r = write_wakealarm("0");
+        r = rtc_read_time(&cmp_time);
         if (r < 0)
                 return r;
 
         log_debug("Woke up at %"PRIu64, cmp_time);
 
-        /* if woken up after alarm time, hibernate */
-        if (cmp_time >= orig_time)
-#if 0 /// elogind uses its manager instance values
-                r = execute(hibernate_modes, hibernate_states);
-#else
-                r = execute(m, "hibernate");
-#endif // 0
+        if (cmp_time < wake_time) /* We woke up before the alarm time, we are done. */
+                return 0;
 
-        return r;
+        /* If woken up after alarm time, hibernate */
+#if 0 /// elogind uses its manager instance values
+        r = execute(hibernate_modes, hibernate_states);
+#else
+        r = execute(m, "hibernate");
+#endif // 0
+        if (r < 0) {
+                log_notice("Couldn't hibernate, will try to suspend again.");
+#if 0 /// elogind uses its manager instance values
+                r = execute(suspend_modes, suspend_states);
+#else
+                r = execute(m, "suspend");
+#endif // 0
+                if (r < 0) {
+                        log_notice("Could neither hibernate nor suspend again, giving up.");
+                        return r;
+                }
+        }
+
+        return 0;
 }
 
 #if 0 /// elogind calls execute() by itself and does not need another binary
-static void help(void) {
+static int help(void) {
+        _cleanup_free_ char *link = NULL;
+        int r;
+
+        r = terminal_urlify_man("systemd-suspend.service", "8", &link);
+        if (r < 0)
+                return log_oom();
+
         printf("%s COMMAND\n\n"
                "Suspend the system, hibernate the system, or both.\n\n"
-               "Commands:\n"
-               "  -h --help            Show this help and exit\n"
-               "  --version            Print version string and exit\n"
-               "  suspend              Suspend the system\n"
-               "  hibernate            Hibernate the system\n"
-               "  hybrid-sleep         Both hibernate and suspend the system\n"
+               "  -h --help              Show this help and exit\n"
+               "  --version              Print version string and exit\n"
+               "\nCommands:\n"
+               "  suspend                Suspend the system\n"
+               "  hibernate              Hibernate the system\n"
+               "  hybrid-sleep           Both hibernate and suspend the system\n"
                "  suspend-then-hibernate Initially suspend and then hibernate\n"
-               "                       the system after a fixed period of time\n"
-               , program_invocation_short_name);
+               "                         the system after a fixed period of time\n"
+               "\nSee the %s for details.\n"
+               , program_invocation_short_name
+               , link
+        );
+
+        return 0;
 }
 
 static int parse_argv(int argc, char *argv[]) {
@@ -368,8 +416,7 @@ static int parse_argv(int argc, char *argv[]) {
         while ((c = getopt_long(argc, argv, "h", options, NULL)) >= 0)
                 switch(c) {
                 case 'h':
-                        help();
-                        return 0; /* done */
+                        return help();
 
                 case ARG_VERSION:
                         return version();
@@ -381,55 +428,47 @@ static int parse_argv(int argc, char *argv[]) {
                         assert_not_reached("Unhandled option");
                 }
 
-        if (argc - optind != 1) {
-                log_error("Usage: %s COMMAND",
-                          program_invocation_short_name);
-                return -EINVAL;
-        }
+        if (argc - optind != 1)
+                return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
+                                       "Usage: %s COMMAND",
+                                       program_invocation_short_name);
 
         arg_verb = argv[optind];
 
-        if (!streq(arg_verb, "suspend") &&
-            !streq(arg_verb, "hibernate") &&
-            !streq(arg_verb, "hybrid-sleep") &&
-            !streq(arg_verb, "suspend-then-hibernate")) {
-                log_error("Unknown command '%s'.", arg_verb);
-                return -EINVAL;
-        }
+        if (!STR_IN_SET(arg_verb, "suspend", "hibernate", "hybrid-sleep", "suspend-then-hibernate"))
+                return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
+                                       "Unknown command '%s'.", arg_verb);
 
         return 1 /* work to do */;
 }
 
-int main(int argc, char *argv[]) {
+static int run(int argc, char *argv[]) {
         bool allow;
         _cleanup_strv_free_ char **modes = NULL, **states = NULL;
         usec_t delay = 0;
         int r;
 
-        log_set_target(LOG_TARGET_AUTO);
-        log_parse_environment();
-        log_open();
+        log_setup_service();
 
         r = parse_argv(argc, argv);
         if (r <= 0)
-                goto finish;
+                return r;
 
         r = parse_sleep_config(arg_verb, &allow, &modes, &states, &delay);
         if (r < 0)
-                goto finish;
+                return r;
 
-        if (!allow) {
-                log_error("Sleep mode \"%s\" is disabled by configuration, refusing.", arg_verb);
-                return EXIT_FAILURE;
-        }
+        if (!allow)
+                return log_error_errno(SYNTHETIC_ERRNO(EACCES),
+                                       "Sleep mode \"%s\" is disabled by configuration, refusing.",
+                                       arg_verb);
 
         if (streq(arg_verb, "suspend-then-hibernate"))
-                r = execute_s2h(delay);
+                return execute_s2h(delay);
         else
-                r = execute(modes, states);
-finish:
-        return r < 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+                return execute(modes, states);
 }
+DEFINE_MAIN_FUNCTION(run);
 #else
 int do_sleep(Manager *m, const char *verb) {
         assert(verb);
