@@ -20,6 +20,7 @@
 #include "rm-rf.h"
 #include "string-util.h"
 #include "strv.h"
+#include "tests.h"
 
 static int here = 0, here2 = 0, here3 = 0;
 void *ignore_stdout_args[] = {&here, &here2, &here3};
@@ -346,7 +347,8 @@ static void test_environment_gathering(void) {
         /* now retest with "default" path passed in, as created by
          * manager_default_environment */
         env = strv_free(env);
-        env = strv_new("PATH=" DEFAULT_PATH, NULL);
+        env = strv_new("PATH=" DEFAULT_PATH);
+        assert_se(env);
 
         r = execute_directories(dirs, DEFAULT_TIMEOUT_USEC, gather_environment, args, NULL, env);
         assert_se(r >= 0);
@@ -365,9 +367,7 @@ static void test_environment_gathering(void) {
 #endif // 0
 
 int main(int argc, char *argv[]) {
-        log_set_max_level(LOG_DEBUG);
-        log_parse_environment();
-        log_open();
+        test_setup_logging(LOG_DEBUG);
 
         test_execute_directory(true);
         test_execute_directory(false);
